--[[ ---------------------------------------------------------------------------
Name:	SimpleRaidFrameOverlays
Author:	Gallowbraid of Terokkar
About:	A very simple enhancement of the built-in raid frames that allows
        overlay colors to be displayed on raid frames for units that have 
        any sort of debuff. This mimics the functionality of almost all raid
        frame addons without much overhead.
----------------------------------------------------------------------------- ]]

local SRFO = LibStub("AceAddon-3.0"):GetAddon("SimpleRaidFrameOverlays")
local L = LibStub("AceLocale-3.0"):GetLocale("SimpleRaidFrameOverlays")
local H = SRFO:NewModule("Highlight", "AceEvent-3.0")

local tempAddFilterName, tempRemoveFilterIndex


SRFO.defaults.global.highlight = {
	multiplehighlights = 1,
	highlightparty = true,
	highlightpartypets = false,
	highlightraid = true,
	highlightraidpets = false,
	debuff = {
		onlydispellable = false,
		ignoreunknown = false,
		curse = {
			color = {1.0, 0.0, 1.0, 0.5},
		},
		disease = {
			color = {0.521568, 0.368627, 0.258823, 0.5,},
		},
		magic = {
			color = {0.0, 0.0, 1.0, 0.5,},
		},
		poison = {
			color = {0.0, 1.0, 0.0, 0.5,},
		},
		unknown = {
			color = {1.0, 0.0, 0.0, 0.5},
		},
		filters = {},
	},
	custom = {
		custom1 = {
			filters = {},
			color = {1.0, 0.5, 0.2, 0.5,},
		},
		custom2 = {
			filters = {},
			color = {0.0, 1.0, 1.0, 0.5,},
		},
		custom3 = {
			filters = {},
			color = {1.0, 1.0, 0.0, 0.5,},
		},
	},
	test = {
		curse = false,
		disease = false,
		magic = false,
		poison = false,
		unknown = false,
		custom1 = false,
		custom2 = false,
		custom3 = false,
	},
}

SRFO.options.args.highlight = {
	type = 'group',
	name = L["Highlight"],
	args = {
		enable = {
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable"],
			get = function(info) return H:IsEnabled() end,
			set = function(info, value) if value then H:Enable() else H:Disable() end end,
			width = "full",
		},
		multiplehighlights = {
			type = 'toggle',
			name = L["MultipleHighlights"],
			desc = L["MultipleHighlightsDesc"],
			get = function(info) return info.handler.db.global.highlight.multiplehighlights end,
			set = function(info, value) 
				info.handler.db.global.highlight.multiplehighlights = value
				H:UpdateForTesting()
				SRFO:UnitFramesChanged()
			end,
			width = "full",
		},
		highlightparty = {
			type = 'toggle',
			name = L["HighlightPartyFrames"],
			desc = L["HighlightPartyFramesDesc"],
			get = function(info) return info.handler.db.global.highlight.highlightparty end,
			set = function(info, value)
				info.handler.db.global.highlight.highlightparty = value
				H:CleanFrames()
				H:UpdateAddonStatus()
				H:UpdateForTesting()
				SRFO:UnitFramesChanged()
			end,
			width = "full",
		},
		highlightpartypets = {
			type = 'toggle',
			name = L["HighlightPartyPetFrames"],
			desc = L["HighlightPartyPetFramesDesc"],
			get = function(info) return info.handler.db.global.highlight.highlightpartypets end,
			set = function(info, value)
				info.handler.db.global.highlight.highlightpartypets = value
				H:CleanFrames()
				H:UpdateAddonStatus()
				H:UpdateForTesting()
				SRFO:UnitFramesChanged()
			end,
			width = "full",
		},
		highlightraid = {
			type = 'toggle',
			name = L["HighlightRaidFrames"],
			desc = L["HighlightRaidFramesDesc"],
			get = function(info) return info.handler.db.global.highlight.highlightraid end,
			set = function(info, value)
				info.handler.db.global.highlight.highlightraid = value
				H:CleanFrames()
				H:UpdateAddonStatus()
				H:UpdateForTesting()
				SRFO:UnitFramesChanged()
			end,
			width = "full",
		},
		highlightraidpets = {
			type = 'toggle',
			name = L["HighlightRaidPetFrames"],
			desc = L["HighlightRaidPetFramesDesc"],
			get = function(info) return info.handler.db.global.highlight.highlightraidpets end,
			set = function(info, value)
				info.handler.db.global.highlight.highlightraidpets = value
				H:CleanFrames()
				H:UpdateAddonStatus()
				H:UpdateForTesting()
				SRFO:UnitFramesChanged()
			end,
			width = "full",
		},
		debuffs = {
			type = 'group',
			name = L["Debuffs"],
			order = 1,
			args = {
				onlydispellable = {
					type = 'toggle',
					name = L["OnlyDispellable"],
					desc = L["OnlyDispellableDesc"],
					get = function(info) return info.handler.db.global.highlight.debuff.onlydispellable end,
					set = function(info, value)
						info.handler.db.global.highlight.debuff.onlydispellable = value
						SRFO:UnitFramesChanged()
					end,
					order = 0,
					width = "full",
				},
				ignoreunknown = {
					type = 'toggle',
					name = L["IgnoreUnknownDebuffs"],
					desc = L["IgnoreUnknownDebuffsDesc"],
					get = function(info) return info.handler.db.global.highlight.debuff.ignoreunknown end,
					set = function(info, value)
						info.handler.db.global.highlight.debuff.ignoreunknown = value
						SRFO:UnitFramesChanged()
					end,
					order = 1,
					width = "full",
				},
				cursecolor = {
					type = 'color',
					name = L["CurseHighlightColor"],
					desc = L["CurseHighlightColorDesc"],
					hasAlpha = true,
					get = function(info) return unpack(info.handler.db.global.highlight.debuff.curse.color) end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.debuff.curse.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 2,
					width = "full",
				},
				diseasecolor = {
					type = 'color',
					name = L["DiseaseHighlightColor"],
					desc = L["DiseaseHighlightColorDesc"],
					hasAlpha = true,
					get = function(info) return unpack(info.handler.db.global.highlight.debuff.disease.color) end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.debuff.disease.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 3,
					width = "full",
				},
				poisoncolor = {
					type = 'color',
					name = L["PoisonHighlightColor"],
					desc = L["PoisonHighlightColorDesc"],
					hasAlpha = true,
					get = function(info) return unpack(info.handler.db.global.highlight.debuff.poison.color) end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.debuff.poison.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 4,
					width = "full",
				},
				magiccolor = {
					type = 'color',
					name = L["MagicHighlightColor"],
					desc = L["MagicHighlightColorDesc"],
					hasAlpha = true,
					get = function(info) return unpack(info.handler.db.global.highlight.debuff.magic.color) end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.debuff.magic.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 5,
					width = "full",
				},
				unknowncolor = {
					type = 'color',
					name = L["UnknownDebuffColor"],
					desc = L["UnknownDebuffColorDesc"],
					hasAlpha = true,
					get = function(info) return unpack(info.handler.db.global.highlight.debuff.unknown.color) end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.debuff.unknown.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 6,
					width = "full",
				},
				filters = {
					type = 'input',
					name = L["DebuffFilters"],
					desc = L["DebuffFiltersDesc"],
					get = function(info)
						return table.concat(info.handler.db.global.highlight.debuff.filters, "\n")
					end,
					set = function(info, value)
						info.handler.db.global.highlight.debuff.filters = {strsplit("\n", value)}
						SRFO:UnitFramesChanged()
					end,
					multiline = true,
					order = 7,
					width = "full",
				},
			},
		},
		custom = {
			type = 'group',
			name = L["Custom"],
			order = 2,
			args = {
				customheading1 = {
					type = 'header',
					name = L["CustomHighlightHeading"] .. " 1",
					order = 7,
				},
				customfilter1 = {
					type = 'input',
					name = L["CustomHighlightFilters"],
					desc = L["CustomHighlightFiltersDesc"],
					get = function(info)
						return table.concat(info.handler.db.global.highlight.custom.custom1.filters)
					end,
					set = function(info, value)
						info.handler.db.global.highlight.custom.custom1.filters = {strsplit("\n", value)}
						SRFO:UnitFramesChanged()
					end,
					multiline = true,
					width = "full",
					order = 8,
				},
				customcolor1 = {
					type = 'color',
					name = L["CustomHighlightColor"],
					desc = L["CustomHighlightColorDesc"],
					hasAlpha = true,
					get = function(info)
						return unpack(info.handler.db.global.highlight.custom.custom1.color)
					end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.custom.custom1.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 9,
				},
				customheading2 = {
					type = 'header',
					name = L["CustomHighlightHeading"] .. " 2",
					order = 10,
				},
				customfilter2 = {
					type = 'input',
					name = L["CustomHighlightFilters"],
					desc = L["CustomHighlightFiltersDesc"],
					get = function(info)
						return table.concat(info.handler.db.global.highlight.custom.custom2.filters)
					end,
					set = function(info, value)
						info.handler.db.global.highlight.custom.custom2.filters = {strsplit("\n", value)}
						SRFO:UnitFramesChanged()
					end,
					multiline = true,
					width = "full",
					order = 11,
				},
				customcolor2 = {
					type = 'color',
					name = L["CustomHighlightColor"],
					desc = L["CustomHighlightColorDesc"],
					hasAlpha = true,
					get = function(info)
						return unpack(info.handler.db.global.highlight.custom.custom2.color)
					end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.custom.custom2.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 12,
				},
				customheading3 = {
					type = 'header',
					name = L["CustomHighlightHeading"] .. " 3",
					order = 13,
				},
				customfilter3 = {
					type = 'input',
					name = L["CustomHighlightFilters"],
					desc = L["CustomHighlightFiltersDesc"],
					get = function(info)
						return table.concat(info.handler.db.global.highlight.custom.custom3.filters)
					end,
					set = function(info, value)
						info.handler.db.global.highlight.custom.custom3.filters = {strsplit("\n", value)}
						SRFO:UnitFramesChanged()
					end,
					multiline = true,
					width = "full",
					order = 14,
				},
				customcolor3 = {
					type = 'color',
					name = L["CustomHighlightColor"],
					desc = L["CustomHighlightColorDesc"],
					hasAlpha = true,
					get = function(info)
						return unpack(info.handler.db.global.highlight.custom.custom3.color)
					end,
					set = function(info, r, g, b, a)
						local color = info.handler.db.global.highlight.custom.custom3.color
						color[1], color[2], color[3], color[4] = r, g, b, a
						H:UpdateForTesting()
						SRFO:UnitFramesChanged()
					end,
					order = 15,
				},
			},
		},
		test = {
			type = 'group',
			name = L["Testing"],
			order = 3,
			args = {
				curse = {
					type = 'toggle',
					name = L["TestCurseHighlight"],
					desc = L["TestCurseHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.curse end,
					set = function(info, value)
						info.handler.db.global.highlight.test.curse = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 1,
				},
				disease = {
					type = 'toggle',
					name = L["TestDiseaseHighlight"],
					desc = L["TestDiseaseHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.disease end,
					set = function(info, value) 
						info.handler.db.global.highlight.test.disease = value 
						H:UpdateForTesting()
					end,
					width = "full",
					order = 2,
				},
				magic = {
					type = 'toggle',
					name = L["TestMagicHighlight"],
					desc = L["TestMagicHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.magic end,
					set = function(info, value)
						info.handler.db.global.highlight.test.magic = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 3,
				},
				poison = {
					type = 'toggle',
					name = L["TestPoisonHighlight"],
					desc = L["TestPoisonHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.poison end,
					set = function(info, value)
						info.handler.db.global.highlight.test.poison = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 4,
				},
				unknown = {
					type = 'toggle',
					name = L["TestUnknownHighlight"],
					desc = L["TestUnknownHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.unknown end,
					set = function(info, value)
						info.handler.db.global.highlight.test.unknown = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 5,
				},
				custom1 = {
					type = 'toggle',
					name = L["TestCustomHighlight"] .. " 1",
					desc = L["TestCustomHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.custom1 end,
					set = function(info, value)
						info.handler.db.global.highlight.test.custom1 = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 6,
				},
				custom2 = {
					type = 'toggle',
					name = L["TestCustomHighlight"] .. " 2",
					desc = L["TestCustomHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.custom2 end,
					set = function(info, value)
						info.handler.db.global.highlight.test.custom2 = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 7,
				},
				custom3 = {
					type = 'toggle',
					name = L["TestCustomHighlight"] .. " 3",
					desc = L["TestCustomHighlightDesc"],
					get = function(info) return info.handler.db.global.highlight.test.custom3 end,
					set = function(info, value)
						info.handler.db.global.highlight.test.custom3 = value
						H:UpdateForTesting()
					end,
					width = "full",
					order = 8,
				},
			},
		},
	},
}

-- Register the events for this module.
function H:OnEnable()
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateAddonStatus")
	if(GetNumPartyMembers() > 0) then
		self:RegisterEvent("UNIT_AURA", "UpdateHighlight")
	end
	if(SRFO:InTestingMode()) then
		self:UpdateForTesting()
	end
end

-- Unregister the events for this module.
function H:OnDisable()
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	self:UnregisterEvent("UNIT_AURA")
end

-- If the player has joined a party, start listening for auras.
function H:UpdateAddonStatus()
	if(GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) then
		self:RegisterEvent("UNIT_AURA", "UpdateHighlight")
		-- Update all of the units with the current auras.
		if(GetNumRaidMembers() > 0) then
			for i=1,25 do
				self:UpdateHighlight("UNIT_AURA", "raid" .. tostring(i))
			end
		elseif(GetNumPartyMembers() > 0) then
			for i=1,4 do
				self:UpdateHighlight("UNIT_AURA", "party" .. tostring(i))
			end
			self:UpdateHighlight("UNIT_AURA", "player")
		end
	else
		self:UnregisterEvent("UNIT_AURA")
	end
end

local OverlayData = {}
local OrphanedFrames = {}
local OrphanFather = CreateFrame("Frame")

-- Updates the highlight on the given unit.
-- @param unit
--   The unit whose highlight to update.
function H:UpdateHighlight(event, unit)
	-- Check to see that the unit is one of the valid units we're going to put an overlay on.
	local unitType = 0 -- 1-party, 2-partypet, 3-raid, 4-raidpet
	if(SRFO.db.global.highlight.highlightparty and SRFO:IsPartyUnit(unit)) then
		unitType = 1
	elseif(SRFO.db.global.highlight.highlightpartypets and SRFO:IsPartyPetUnit(unit)) then
		unitType = 2
	elseif(SRFO.db.global.highlight.highlightraid and SRFO:IsRaidUnit(unit)) then
		unitType = 3
	elseif(SRFO.db.global.highlight.highlightraidpets and SRFO:IsRaidPetUnit(unit)) then
		unitType = 4
	end
	if(unitType > 0) then
		-- Check to see what debuffs the unit has
		local check = 0
		if(SRFO.db.global.highlight.debuff.onlydispellable) then check = 1 end
		local typeFlags = {}
		-- If we're in testing mode, use the testing flags.
		if(event == "TEST") then
			typeFlags["Curse"] = SRFO.db.global.highlight.test.curse
			typeFlags["Disease"] = SRFO.db.global.highlight.test.disease
			typeFlags["Magic"] = SRFO.db.global.highlight.test.magic
			typeFlags["Poison"] = SRFO.db.global.highlight.test.poison
			typeFlags["Unknown"] = SRFO.db.global.highlight.test.unknown
			typeFlags["Custom1"] = SRFO.db.global.highlight.test.custom1
			typeFlags["Custom2"] = SRFO.db.global.highlight.test.custom2
			typeFlags["Custom3"] = SRFO.db.global.highlight.test.custom3
		-- Otherwise, loop through all of the debuffs and analyze each one.
		else
			for i=1,40 do
				local name, rank, icon, count, debuffType = UnitDebuff(unit, i, check)
				if(name) then
					-- Check to see if it is in a custom filter
					if(H:IsCustom1(name)) then
						typeFlags["Custom1"] = true
					elseif(H:IsCustom2(name)) then
						typeFlags["Custom2"] = true
					elseif(H:IsCustom3(name)) then
						typeFlags["Custom3"] = true
					-- Check to see if it is not a filtered debuff.
					elseif not self:IsDebuffFiltered(name) then
						-- If unkonwn, only highlight if enabled.
						if(not debuffType) then
							if(not SRFO.db.global.highlight.debuff.ignoreunknown) then 
								typeFlags["Unknown"] = true
							end
						else
							typeFlags[debuffType] = true
						end
					end
				else
					break
				end
			end
			for i=1,40 do
				local name = UnitBuff(unit, i)
				if(name) then
					-- Check to see if it is in a custom filter
					if(H:IsCustom1(name)) then
						typeFlags["Custom1"] = true
					elseif(H:IsCustom2(name)) then
						typeFlags["Custom2"] = true
					elseif(H:IsCustom3(name)) then
						typeFlags["Custom3"] = true
					end
				else
					break
				end
			end
		end
		-- Build a list of colors for frames that we'll have to create
		local colors = {}
		if(typeFlags["Curse"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.debuff.curse.color)}
		end
		if(typeFlags["Disease"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.debuff.disease.color)}
		end
		if(typeFlags["Magic"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.debuff.magic.color)}
		end
		if(typeFlags["Poison"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.debuff.poison.color)}
		end
		if(typeFlags["Unknown"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.debuff.unknown.color)}
		end
		if(typeFlags["Custom1"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.custom.custom1.color)}
		end
		if(typeFlags["Custom2"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.custom.custom2.color)}
		end
		if(typeFlags["Custom3"]) then
			colors[#colors+1] = {unpack(SRFO.db.global.highlight.custom.custom3.color)}
		end
		
		-- If the setting for multiple colors is disabled and there are multiple colors,
		-- force it to the unknown color.
		if(not SRFO.db.global.highlight.multiplehighlights and #colors > 1) then
			colors = {}
			colors[1] = {unpack(SRFO.db.global.highlight.debuff.unknown.color)}
		end
		-- Create more overlay frames until the number of overlay frames for the unit
		-- frame matches the number of debuffs.
		local overlayData = OverlayData[unit]
		local frames = OverlayData.frames
		while(#colors > self:GetNumOverlayFrames(unit)) do
			self:AddOverlayFrame(unit, unitType)
		end
		-- If there are more overlay frames than debuffs on the unit, clear the overlay
		-- frames until the number matches.
		while(#colors < self:GetNumOverlayFrames(unit)) do
			self:RemoveLastOverlayFrame(unit)
		end
		-- If there are actually highlights, handle them.
		if(#colors > 0) then
			local uf = SRFO:GetUnitFrame(unit, unitType)
			-- Now that the number of overlay frames matches, resize and position them 
			-- to account for the number of highlights and set the background color.
			local width = uf:GetWidth()
			local ovWidth = width/#colors
			for i=1,#colors do
				local f = self:GetOverlayFrame(unit, i)
				f:SetHeight(uf:GetHeight())
				f:SetWidth(ovWidth)
				f:SetPoint("TOPLEFT", uf, "TOPLEFT", (i-1)*ovWidth, 0.0)
				f:SetBackdropColor(colors[i][1], colors[i][2], colors[i][3], colors[i][4])
				f:SetBackdropBorderColor(colors[i][1], colors[i][2], colors[i][3], 1.0)
			end
		end
	end
end

-- Function called by the addon to clean up any current overlays.
-- This function removes all of the highlight frames from unit frames and stores them in
-- the orphaned list for later use.
function H:CleanFrames()
	for k,v in pairs(OverlayData) do
		for k,v in pairs(v.frames) do
			v:SetParent(OrphanFather)
			OrphanedFrames[#OrphanedFrames+1] = v
			v:Hide()
		end
		v.frames = {}
	end
end

-- Function called by either the addon or the module's testing options changing.
function H:UpdateForTesting()
	if(SRFO:InTestingMode()) then
		self:UpdateHighlight("TEST", "testparty")
		self:UpdateHighlight("TEST", "testpartypet")
		self:UpdateHighlight("TEST", "testraid")
		self:UpdateHighlight("TEST", "testraidpet")
	else
		self:CleanFrames()
		self:UpdateAddonStatus()
	end
end

function H:IsDebuffFiltered(debuff)
	for k,v in pairs(SRFO.db.global.highlight.debuff.filters) do
		if(debuff == v) then return true end
	end
	return false
end

function H:IsCustom1(debuff)
	for k,v in pairs(SRFO.db.global.highlight.custom.custom1.filters) do
		if(debuff == v) then return true end
	end
	return false
end

function H:IsCustom2(debuff)
	for k,v in pairs(SRFO.db.global.highlight.custom.custom2.filters) do
		if(debuff == v) then return true end
	end
	return false
end

function H:IsCustom3(debuff)
	for k,v in pairs(SRFO.db.global.highlight.custom.custom3.filters) do
		if(debuff == v) then return true end
	end
	return false
end

function H:GetNumOverlayFrames(unit)
	if(OverlayData[unit] and OverlayData[unit].frames) then
		return #(OverlayData[unit].frames)
	end
	return 0
end

local backdrop = {
	bgFile = "Interface\\Buttons\\WHITE8X8",
	edgeFile = "Interface\\Buttons\\WHITE8X8",
	tile = false,
	tileSize = 0,
	edgeSize = 2,
	insets = { left=0, right=0, top=0, bottom=0 },
}

function H:AddOverlayFrame(unit, unitType)
	local uf = SRFO:GetUnitFrame(unit, unitType)
	if(not OverlayData[unit]) then
		OverlayData[unit] = {
			frames = {}
		}
	end
	local frames = OverlayData[unit].frames
	if(#OrphanedFrames > 0) then
		local of = OrphanedFrames[#OrphanedFrames]
		of:SetParent(uf)
		frames[#frames+1] = of
		table.remove(OrphanedFrames, #OrphanedFrames)
	else
		frames[#frames+1] = CreateFrame("Frame", "SRFOHighlightFrame" .. tostring(#frames+1), uf)
	end
--	frames[#frames]:SetLevel(uf:GetLevel()+1)
	frames[#frames]:SetBackdrop(backdrop)
	frames[#frames]:Show()
end

function H:RemoveLastOverlayFrame(unit)
	local frames = OverlayData[unit].frames
	frames[#frames]:SetParent(OrphanFather)
	frames[#frames]:Hide()
	OrphanedFrames[#OrphanedFrames+1] = frames[#frames]
	table.remove(frames, #frames)
end

function H:GetOverlayFrame(unit, frame)
	return OverlayData[unit].frames[frame]
end
